#!/usr/bin/env python
# -*- coding: utf-8 -*-

# GIMP plugin to create a drawing look 

# (c) Dinasset 2021
#
#   History:
#
#   v1.0: 2021-02-28: Initial version
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

 
from gimpfu import *
import random
from random import randint
import sys, os
import time
import datetime

# define here some constant value, when required
FixWH=3500

############################################################################
# include, then use: erro_msg or info_msg
#---------------------------------------------------------------------------
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    gimp.message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    gimp.message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###

############################################################################
# service functions
#---------------------------------------------------------------------------
   
def PaletteFromImage (image,layer,iname,nr_of_colours):
    
    # remove previously generated palettes
    Pname="Palette generated by "+iname
    PnameALL="Palette generated by "
    (nP,lP)=pdb.gimp_palettes_get_list(PnameALL)
    if nP>0:
        x=0
        for x in range(0,nP): 
            pdb.gimp_palette_delete (lP[x])
        gimp.message("deleted "+str(nP)+" palette(s) generated in previous run(s)")    
        
    PaletteImage = pdb.gimp_image_new (image.width, image.height, RGB)                    
    PaletteBG = pdb.gimp_layer_new_from_drawable (layer, PaletteImage)
    PaletteImage.add_layer (PaletteBG, 0)
    PaletteBG.mode = LAYER_MODE_NORMAL
    
    pdb.plug_in_gmic_qt(PaletteImage, PaletteBG, 1,2 ,
    #[G'MIC] Posterize: fx_posterize 150,30,1,12,0,0,0,0,50,50
    "-v - -fx_posterize 150,30,1,"+str(nr_of_colours)+",0,0,1,0,50,50 ")
    PosterLayer = PaletteImage.layers[0]
    PosterLayer.mode = LAYER_MODE_NORMAL
    PosterLayer.opacity = 100.0  
    PosterLayer.name = "[GMIC] -fx_posterize 150,30,1,"+str(nr_of_colours)+",0,0,0,0,50,50 "
    
    pdb.plug_in_gmic_qt(PaletteImage, PosterLayer, 1,2 ,
    #[G'MIC] Colors to Layers: fx_split_colors 0,256,0,0
    "-v - -fx_split_colors 0,256,0,0 ")
    (num_layers,layer_ids) = pdb.gimp_image_get_layers (PaletteImage)
    generated = num_layers-2    # PaletteBG and PosterLayer
    #gimp.message ("generated "+str(generated)+" layers")
    
    areaNR=0
    area_color=[]   # empty list
    for areaNR in range(0,generated):
        #one_color=pdb.gimp_palette_entry_get_color(temp_palette, areaNR)
        pdb.gimp_context_set_antialias(False)
        pdb.gimp_context_set_feather(False)
        pdb.gimp_context_set_sample_criterion(SELECT_CRITERION_COMPOSITE)
        pdb.gimp_context_set_sample_merged(False)
        pdb.gimp_context_set_sample_threshold(0.0)
        pdb.gimp_context_set_sample_transparent(True)
        # select transparent
        pdb.gimp_image_select_color(PaletteImage, CHANNEL_OP_REPLACE,
                                    PaletteImage.layers[areaNR], (0,0,0,0)) 
        pdb.gimp_selection_invert (PaletteImage)
        (there_is_a_selection, x1, y1, x2, y2) = pdb.gimp_selection_bounds(PaletteImage)
        if there_is_a_selection==True:
            (meanR,B,C,D,E,F) = pdb.gimp_drawable_histogram (PaletteImage.layers[areaNR],
                                                         HISTOGRAM_RED, 0.0, 1.0)
            (meanG,B,C,D,E,F) = pdb.gimp_drawable_histogram (PaletteImage.layers[areaNR],
                                                         HISTOGRAM_GREEN, 0.0, 1.0)
            (meanB,B,C,D,E,F) = pdb.gimp_drawable_histogram (PaletteImage.layers[areaNR],
                                                         HISTOGRAM_BLUE, 0.0, 1.0)
            color = (int(meanR),int(meanG),int(meanB))  # WARNING: int() is necessary !
            area_color.append(color)
            #gimp.message ("color for area "+str(areaNR)+" is "+str(color))
        else:
            gimp.message ("area nr. "+str(areaNR)+" there_is_NO_selection")
            
    pdb.gimp_selection_none(PaletteImage)
    
    generated_palette = pdb.gimp_palette_new (Pname)
    for areaNR in range(0,generated):
        pdb.gimp_palette_add_entry(generated_palette, str(areaNR), area_color[areaNR])
    #info_msg ("created: "+str(generated_palette))
    
    pdb.gimp_palettes_refresh()
    
    posterized=pdb.gimp_image_flatten (PaletteImage)
    posterized.name = iname+", posterized with palette creation" 
    
    return (posterized,generated_palette)

####################
#   M A I N
#-------------------
    
def DrawStyleR (inImage, inDrawable, widthheight,
                inAreas, inOpacity, postProc,
                doFlatten) :
    
    pdb.gimp_context_push()
    currentFG = pdb.gimp_context_get_foreground()
    # source image characteristics
    owidth = inImage.width
    oheight = inImage.height
    OrigWH = owidth+oheight
    otype = inImage.base_type
    # image name
    iname = inImage.name
    split = iname.split("(", 1)
    iname = split[0]
    end = len(iname)-2
    iname = iname[1:end]    # remove []
    #info_msg (iname)
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime('%Y-%m-%d %H-%M-%S')
    ymdhms = datetime.datetime.fromtimestamp(ts).strftime('%Y%m%d%H%M%S')
    #info_msg ("plasma #"+ymdhms[8:])
    
    Ratio = FixWH*1.00/OrigWH*1.00
    rWidth = int(owidth*Ratio)
    rHeight = int(oheight*Ratio)
    
    # START    
    # create output image
    newImage = pdb.gimp_image_new (owidth, oheight, otype)
    newBG = pdb.gimp_layer_new_from_drawable (inDrawable, newImage)
    newImage.add_layer (newBG, 0)
    newBG.name = inDrawable.name

    name = newBG.name
    if '.' in name:
        split = name.split(".", 1)
        name = split[0]
    # ofnuts technique to give it a name (unsaved). I set it to the name of the original layer
    newImage.filename=os.path.join(gimp.user_directory(4),iname)
    
    pdb.gimp_image_scale (newImage, rWidth, rHeight)
    
    #pdb.gimp_display_new (newImage)
    #return()
     
    # First part: Posterize
    
    inNrAreas=int(inAreas)
    posterized_layer = newBG.copy()
    newImage.add_layer (posterized_layer, 0)
    pdb.gimp_drawable_posterize (posterized_layer, inNrAreas)
    posterized_layer.name="Posterized"
    
    #pdb.gimp_display_new (newImage)
    #return()
     
    # Second part: Change colour to a pastel nuance
    normalized_layer=posterized_layer.copy()
    newImage.add_layer (normalized_layer, 0)
    normalized_layer.name="Normalized"
    pdb.gimp_drawable_hue_saturation(normalized_layer,
                                     HUE_RANGE_ALL, 0.0, 0.0, 100.0, 0.0)
    pdb.gimp_drawable_curves_spline (normalized_layer, 0, 6,
                                (0.0,0.15, 0.5,0.5, 1.0,0.85))  
    
    #pdb.gimp_display_new (newImage)
    #return()
     
    # Third part: create the border with DoG
    #              
    borders_layer=posterized_layer.copy()
    newImage.add_layer (borders_layer, 0)
    borders_layer.name="Borders"
    borders_layer.mode=LAYER_MODE_BURN
    borders_layer.opacity=90.0
    innerRadius=4
    outerRadius=2
    pdb.plug_in_dog (newImage, borders_layer,
                     innerRadius, outerRadius, True, True)
    
    pdb.gimp_drawable_curves_spline (borders_layer, 0, 6,
                                (0.0,0.5, 0.8,0.6, 1.0,1.0))  
    pdb.gimp_layer_add_alpha (borders_layer)
    
    pdb.plug_in_spread (newImage, borders_layer, 5.0, 5.0)
    
     
    finalPoster=pdb.gimp_layer_new_from_visible(newImage,newImage,
                                          name+" final poster ")
    newImage.add_layer(finalPoster,0)
    finalPoster.mode=LAYER_MODE_NORMAL
    
    #pdb.gimp_display_new (newImage)
    #pdb.gimp_context_pop()
    #return
     
    # Fourth part: create whirls
    #              
        
    pdb.plug_in_gmic_qt(newImage, finalPoster, 1,2 ,
    #[G'MIC] Whirl Drawing: fx_draw_whirl 100,0,50,50
    "-v - -fx_draw_whirl 100,0,50,50 ")
    Whirl = newImage.layers[0]
    Whirl.name = "Texture by Whirl Drawing"
    #pdb.plug_in_edge (newImage, Whirl, 3.0, 0, 1)
    pdb.plug_in_emboss (newImage, Whirl, 45.0, 45.0, 20.0, 1)
    Whirl.mode = LAYER_MODE_HSV_VALUE
    Whirl.opacity = 90.0
    
    #pdb.gimp_display_new (newImage)
    #pdb.gimp_context_pop()
    #return
    
    strokeMode = "draw whirl, plasma at "+str(int(inOpacity))+"%"
     
    # Fifth part: create plasma
    #              
    Plasma = pdb.gimp_layer_new (newImage, rWidth, rHeight,
                                  RGBA_IMAGE, "Plasma", 90.0,
                                  LAYER_MODE_LCH_HUE)
    newImage.add_layer(Plasma,0)
    pdb.plug_in_plasma (newImage, Plasma, ymdhms[8:], 7.0)
    Plasma.opacity = inOpacity*1.0
    
    #pdb.gimp_display_new (newImage)
    #pdb.gimp_context_pop()
    #return
    
    PreOutcome = pdb.gimp_layer_new_from_visible(newImage,newImage,
                                          " preliminary outcome")
    newImage.add_layer(PreOutcome,0)
    PreOutcome.mode=LAYER_MODE_NORMAL
    PreOutcome.opacity = 100.0
    
    BGonTop = newBG.copy()
    newImage.add_layer(BGonTop,0)
    BGonTop.name = "BG on Top for colours"
    pdb.gimp_drawable_curves_spline (BGonTop, 0, 4, (0.0,0.33, 1.0,0.67))  
    BGonTop.mode = LAYER_MODE_GRAIN_MERGE
    BGonTop.opacity = 75.0
    
    if postProc>0:  # 0=*none
        forPostProcess = pdb.gimp_layer_new_from_visible(newImage,newImage, "Post-process")
        newImage.add_layer(forPostProcess, 0)
        if postProc==1:  # 1=apply curves
            pdb.gimp_drawable_curves_spline (forPostProcess, 0, 8,
                              (0.0,0.0, 0.2,0.1, 0.5,0.5, 0.8,0.9, 1.0,1.0))
            forPostProcess.mode = LAYER_MODE_NORMAL
            forPostProcess.opacity = 100.0
        if postProc==2:  # 2=apply linear light
            forPostProcess.mode = LAYER_MODE_LINEAR_LIGHT
            forPostProcess.opacity = 60.0
        if postProc==3:  # 2=apply linear burn
            forPostProcess.mode = LAYER_MODE_LINEAR_BURN
            forPostProcess.opacity = 60.0
    
    if doFlatten==True:
        pdb.gimp_item_set_visible (newBG, False) # temp
        Outcome = pdb.gimp_image_merge_visible_layers (newImage, CLIP_TO_IMAGE)
        pdb.gimp_item_set_visible (newBG, True) # reset
    else:
        Outcome = pdb.gimp_layer_new_from_visible(newImage,newImage, "")
        newImage.add_layer(Outcome,0)
        pdb.gimp_item_set_visible (Outcome, False)
        
    Outcome.name = st+" "+name+" stroked "+strokeMode
    
    pdb.gimp_display_new(newImage)
    
    # tier down
     
    # calculate scale factor for the new output image
    if widthheight == 0 :   # means the user wants to have same size as input
        widthheight = OrigWH
    if widthheight != FixWH:    
        uscale = widthheight*1.0/FixWH*1.0      # ratio for scaling
        pdb.gimp_image_scale (newImage, width*uscale,
                              height*uscale)   # user chosen size
         
    pdb.gimp_context_pop()
    
    return(Outcome)

# This is the plugin registration function

### Registration
whoiam="\n"+os.path.abspath(sys.argv[0])
author="dinasset"
menu="<Image>/Diego/Drawing/"
copyrightYear="2021"
desc="Draw Effect Style R "

register(
    "DrawStyleR",
    desc+whoiam,desc,
    author,author,
    copyrightYear,
    desc+"...",
    "*",
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "drawable", "Input drawable", None),
      (PF_ADJUSTMENT, "size", "Width+Height of New Image"+
                                "\n (0=same as input)", 3500, (0, 8000, 200)),
      (PF_ADJUSTMENT, "colours", "How many colour areas:", 15, (3, 31, 1)),
      (PF_ADJUSTMENT, "plasma", "Opacity of plasma layer:", 35, (5, 95, 5)),
      (PF_OPTION, "postpro", "Post-process touch-up?"+
                            "\n  (for colours/contrast adjustments)", 0, ["*none",
                                                 "curves","linear light","linear burn"]),
      (PF_TOGGLE, "flatten", "Flatten Image?", True),
    ], 
    [
      (PF_DRAWABLE, "drawable", "Output drawable", None),
    ],
    DrawStyleR,
    menu=menu
    )

main()
